from datetime import datetime

from magento.attributes import (
    get_custom_attribute, get_boolean_custom_attribute, get_custom_attributes_dict,
    set_custom_attribute, set_custom_attributes, serialize_attribute_value
)
from magento.client import Magento
from magento.exceptions import MagentoException, MagentoAssertionError
from magento.order_helpers import is_order_on_hold, is_order_cash_on_delivery, get_order_shipping_address
from magento.queries import Query, make_field_value_query, make_search_query
from magento.version import __version__

# ids of the default categories in Magento
ROOT_CATEGORY_ID = 1
DEFAULT_CATEGORY_ID = 2

# Magento visibility options
# https://devdocs.magento.com/guides/v2.4/rest/tutorials/configurable-product/create-configurable-product.html
VISIBILITY_BOTH = 4
VISIBILITY_IN_CATALOG = 2
VISIBILITY_IN_SEARCH = 3
VISIBILITY_NOT_VISIBLE = 1

# Magento product statuses
# https://magento.stackexchange.com/q/10693/92968
ENABLED_PRODUCT = 1
DISABLED_PRODUCT = 2


def format_datetime(dt: datetime):
    """Format a datetime for Magento."""
    # "2021-07-02 13:19:18.300700" -> "2021-07-02 13:19:18"
    return dt.isoformat(sep=" ").split(".", 1)[0]



import re
import requests
from typing import List, Optional, Union, Dict, cast

RE_PARAM = re.compile(r"%([A-Za-z0-9]+)")

__all__ = ('build_exception_text', 'MagentoException', 'MagentoAssertionError')


def build_exception_text(message: str, parameters: Optional[Union[Dict[str, str], List[str]]]):
    if not parameters:
        return message

    # Magento sometimes return a dict, sometimes a list.
    # For example:
    #   message = "%fieldName is a required field."
    #   parameters = {"fieldName":"product"}
    if isinstance(parameters, dict):
        return cast(str, RE_PARAM.sub(lambda m: str(parameters[m.group(1)]), message))  # type: ignore

    # For example:
    #   message = 'El atributo "%1" no incluye una opción con el ID "%2".'
    #   parameters = ['manufacturer', '17726']
    if isinstance(parameters, list):
        return cast(str, RE_PARAM.sub(lambda m: str(parameters[int(m.group(1)) - 1]), message))  # type: ignore

    # Ok fallback
    return str((message, parameters))


class MagentoException(Exception):
    def __init__(self, message,
                 parameters: Optional[Union[Dict[str, str], List[str]]] = None,
                 trace: Optional[str] = None,
                 response: Optional[requests.Response] = None):
        self.message = message
        self.parameters = parameters
        self.response = response
        self.trace = trace

        text = build_exception_text(message, parameters)
        super().__init__(text)


class MagentoAssertionError(AssertionError):
    """
    Exception raised by ``Magento#get_product_by_query`` when the query returns more than one product.

    This exception doesn’t inherit from ``MagentoException`` because it’s not a Magento error per se.
    """
    pass
